@@ -16,7 +16,12 @@ try:
 except ImportError:
     # Python 2.6
     from .py27_weakrefset import WeakSet
+try:
+    import asyncio
+except ImportError:
+    asyncio = None
 
+from . import compat
 from . import events
 from . import executor
 from . import futures
@@ -25,6 +30,13 @@ from .coroutines import Return, From, coroutine, iscoroutinefunction, iscoroutin
 from . import coroutines
 
 
+if asyncio is not None:
+    # Accept also asyncio Future objects for interoperability
+    _FUTURE_CLASSES = (futures.Future, asyncio.Future)
+else:
+    _FUTURE_CLASSES = futures.Future
+
+
 @coroutine
 def _lock_coroutine(lock):
     yield From(lock.acquire())
@@ -90,7 +102,15 @@ class Task(futures.Future):
         i = res.find('<')
         if i < 0:
             i = len(res)
-        res = res[:i] + '(<{0}>)'.format(self._coro.__name__) + res[i:]
+        text = self._coro.__name__
+        coro = self._coro
+        if iscoroutine(coro):
+            filename = coro.gi_code.co_filename
+            if coro.gi_frame is not None:
+                text += ' at %s:%s' % (filename, coro.gi_frame.f_lineno)
+            else:
+                text += ' done at %s' % filename
+        res = res[:i] + '(<{0}>)'.format(text) + res[i:]
         return res
 
     def get_stack(self, limit=None):
@@ -171,9 +191,9 @@ class Task(futures.Future):
                 print(line, file=file, end='')
 
     def cancel(self):
-        """Request that a task to cancel itself.
+        """Request this task to cancel itself.
 
-        This arranges for a CancellationError to be thrown into the
+        This arranges for a CancelledError to be thrown into the
         wrapped coroutine on the next cycle through the event loop.
         The coroutine then has a chance to clean up or even deny
         the request using try/except/finally.
@@ -221,11 +241,17 @@ class Task(futures.Future):
                 result = coro.send(value)
             else:
                 result = next(coro)
-        except Return as exc:
-            exc.raised = True
-            self.set_result(exc.value)
-        except StopIteration:
-            self.set_result(None)
+        except StopIteration as exc:
+            if compat.PY33:
+                # asyncio Task object? get the result of the coroutine
+                result = exc.value
+            else:
+                if isinstance(exc, Return):
+                    exc.raised = True
+                    result = exc.value
+                else:
+                    result = None
+            self.set_result(result)
         except futures.CancelledError as exc:
             super(Task, self).cancel()  # I.e., Future.cancel(self).
         except Exception as exc:
@@ -252,7 +278,7 @@ class Task(futures.Future):
                 coro = _lock_coroutine(result)
                 result = Task(coro, loop=self._loop)
 
-            if isinstance(result, futures.Future):
+            if isinstance(result, _FUTURE_CLASSES):
                 # Yielded Future must come from Future.__iter__().
                 result.add_done_callback(self._wakeup)
                 self._fut_waiter = result
@@ -285,7 +311,7 @@ class Task(futures.Future):
 
 # wait() and as_completed() similar to those in PEP 3148.
 
-# Export symbols in asyncio.tasks for compatibility with Tulip
+# Export symbols in trollius.tasks for compatibility with Tulip
 FIRST_COMPLETED = executor.FIRST_COMPLETED
 FIRST_EXCEPTION = executor.FIRST_EXCEPTION
 ALL_COMPLETED = executor.ALL_COMPLETED
@@ -295,13 +321,15 @@ ALL_COMPLETED = executor.ALL_COMPLETED
 def wait(fs, loop=None, timeout=None, return_when=ALL_COMPLETED):
     """Wait for the Futures and coroutines given by fs to complete.
 
+    The sequence futures must not be empty.
+
     Coroutines will be wrapped in Tasks.
 
     Returns two sets of Future: (done, pending).
 
     Usage:
 
-        done, pending = yield From(asyncio.wait(fs))
+        done, pending = yield From(trollius.wait(fs))
 
     Note: This does not raise TimeoutError! Futures that aren't done
     when the timeout occurs are returned in the second set.
@@ -339,7 +367,7 @@ def wait_for(fut, timeout, loop=None):
 
     Usage:
 
-        result = yield From(asyncio.wait_for(fut, 10.0))
+        result = yield From(trollius.wait_for(fut, 10.0))
 
     """
     if loop is None:
